#ifndef __BIAWEBSIDEBAR__
#define __BIAWEBSIDEBAR__
#include <string>
#include <list>
#include <iostream>
#include <fstream>
#include <fmt/format.h>
#include "biawebutil.hpp"

// classes to describe the a list of items and sidebar item containers which form part of 
// main document
namespace biaweb {
    // class to represent a sidebar item which can contain a text and link or only
    // text
    class SideBarItem {
      protected:
        // sidebar text and url
        std::string sidebar_text;
        std::string sidebar_url;
      public:
        std::string get_sidebar_text () {
            return this->sidebar_text;
        }
        void set_sidebar_text (std::string text) {
            this->sidebar_text = escape_html (text);
        }
        std::string get_sidebar_url () {
            return this->sidebar_url;
        }
        void set_sidebar_url (std::string url) {
            this->sidebar_url = escape_html (url);
        }
        // output to HTML using a template directory specified
        std::string to_html (std::string templatedir);
    
        SideBarItem (std::string text = "", std::string url = "") {
            this->sidebar_text = escape_html (text);
            this->sidebar_url = escape_html (url);
        }
    };

    std::string SideBarItem::to_html (std::string templatedir) {
        std::string html;
        // if url is not empty it is a link item so load the sidebar link template
        if (! this->sidebar_url.empty ()) {
            if (!this->sidebar_text.empty ())
            {
                std::ifstream tpl_linkitem (templatedir + "/sidebarlinkitem.tpl.html");
                std::string tpl_linkitem_str ( (std::istreambuf_iterator<char> (tpl_linkitem)),
                                                (std::istreambuf_iterator<char> ()));
                tpl_linkitem.close ();
                html = fmt::format (tpl_linkitem_str, 
                                        fmt::arg ("itemurl", this->sidebar_url),
                                        fmt::arg ("itemtext", this->sidebar_text));
            }
            // no text or url - item is empty - so it should be blank
            else
                html = "";
        }
        // Non link item. Load the normal sidebar item template.
        else
        {
            std::ifstream tpl_item (templatedir + "/sidebaritem.tpl.html");
            std::string tpl_item_str ( (std::istreambuf_iterator<char> (tpl_item)),
                                        (std::istreambuf_iterator<char> ()));
            tpl_item.close ();
            html = fmt::format (tpl_item_str, fmt::arg ("itemtext", this->sidebar_text) );
        } 
        return html;
    }
    
    // Class to represent a sidebar, which contains one heading and a list of items 
    // either links or non-link items.
    class SideBar {
      protected:
        std::string sidebar_title;
        std::list <SideBarItem> items;
        
      public:
        void set_title (std::string title) {
            this->sidebar_title = title;
        }

        void add_sidebar_item (SideBarItem item) {
            this->items.insert (this->items.cend(), item);
        }

        // render the sidebar using the template directory specified
        std::string to_html (std::string templatedir) ;
    };

    // render the sidebar to HTML representation from the template.
    std::string SideBar::to_html (std::string templatedir) {
        std::ifstream sidetpl (templatedir + "/sidebar.tpl.html");
        std::string sidetpl_str ( ( std::istreambuf_iterator<char> (sidetpl)) , 
                                       (std::istreambuf_iterator<char> ()));
        sidetpl.close ();
        std::string listitems;
        // first get the sidebar items and render them to HTML
        for (SideBarItem item : this->items) {
            listitems += item.to_html (templatedir);
        }
        
        std::string html = "";
        // if there are items, sidebar should be rendered otherwise not needed
        if (items.size () > 0)
            html = fmt::format (sidetpl_str, 
                                        fmt::arg ("title", this->sidebar_title),
                                        fmt::arg ("items", listitems));

        return html; 
    }
}

#endif
